package com.lotadata.geo_sdk_flutter_plugin;

import android.Manifest;
import android.annotation.TargetApi;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.media.RingtoneManager;
import android.os.Build;
import android.preference.PreferenceManager;
import android.widget.Toast;

import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.lotadata.moments.ConnectionResult;
import com.lotadata.moments.Moments;
import com.lotadata.moments.MomentsClient;
import com.lotadata.moments.TrackingMode;
import com.lotadata.moments.engagements.NotificationReceiver;
import com.lotadata.moments.events.LDEvent;
import com.lotadata.moments.events.LDEventListener;
import com.lotadata.moments.events.LDEventType;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import io.flutter.plugin.common.MethodChannel;

import static java.sql.DriverManager.println;
import static java.util.logging.Logger.global;

public class GeoSDK {

    private  Activity mActivity;
    private MethodChannel mMethodChannel;

    private int PERMISSIONS_REQUEST = 100;
    private String[] mPermissions = {Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_COARSE_LOCATION};

    private static final String PAYLOAD_COUNT_PREFS = "payloadCount.prefs";
    private static final String PAYLOAD_COUNT_KEY = "payloadCount";
    private Moments mMomentsClient;

    private  MomentsClient.ConnectionListener mConnectionListener;

    private LDEventListener mEventListener = null;
    private final Object mLock = new Object();
    private LDEventType lastEventType = LDEventType.UNKNOWN;

    private List<LDEvent> eventsHistory = new ArrayList<LDEvent>();

    protected BroadcastReceiver payloadReceiver = null;

    protected  int count = 0;


    public GeoSDK(Activity activity, MethodChannel methodChannel) {
        this.mActivity = activity;
        this.mMethodChannel = methodChannel;

    }


    public void checkPermissionsAndLaunch() {

        try{

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (!hasPermissions()) {
                    mActivity.requestPermissions(mPermissions, PERMISSIONS_REQUEST);
                }
            }

            initializeLotaDataSDK();

        } catch (Exception ex) {
            System.out.println("GEO_SDK_FLUTTER_PLUGIN: - Exception:");
            System.out.println(ex.getMessage());
        }

    }


    private void initializeLotaDataSDK() {
        try {

            count = getPastCount();

            trackPayloads();

            try {

                MomentsClient.getInstance(mActivity,  new MomentsClient.ConnectionListener() {
                    @Override
                    public void onConnected(Moments client) {

                        mMomentsClient = client;
                        mMomentsClient.setBgTrackingMode(TrackingMode.STAY_DETECTION);
                        mMomentsClient.setFgTrackingMode(TrackingMode.HAWK_EYE);
                        trackEvents();
//                        trackEngagements();


                    }

                    @Override
                    public void onConnectionError(ConnectionResult result) {
                        System.out.println("GEO_SDK_FLUTTER_PLUGIN: ERROR WHILE CONNECTING");
                    }

                });

            } catch (Exception ex) {
                System.out.println("GEO_SDK_FLUTTER_PLUGIN: Exception - "  + ex.getMessage());
            }

        } catch (Exception ex) {
            throw ex;
        }


    }

    protected void trackEvents() {

        if(mMomentsClient != null) mMomentsClient.trackEvents(mEventListener = new LDEventListener() {
            @Override
            public void onEvent(LDEvent ldEvent) {


                synchronized (mLock) {

                    if((ldEvent == null) || (ldEvent.type == null) || (ldEvent.type == lastEventType)) return;

                    /**
                     * Sends the event to [onEvent] handler in dart code
                     */
                    mMethodChannel.invokeMethod("onEvent", ldEvent);

                    updateEventsHistory(ldEvent);

                    System.out.println("GEO_SDK_FLUTTER_PLUGIN: Type of Event Triggered - " + ldEvent.type);
                }

            }
        });
    }


    /**
     * Get last event
     */
    public LDEvent getLastEvent() {
        synchronized (eventsHistory) {
            if(eventsHistory.size() == 0) return null;
            return eventsHistory.get(eventsHistory.size() - 1);
        }
    }

    private void updateEventsHistory(LDEvent ldEvent) {

            eventsHistory.add(ldEvent);

            //Persist in shared preference
            SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(mActivity.getApplicationContext());
            Type listOfEvents = new TypeToken<List<LDEvent>>() {}.getType();
            String strEvents = new Gson().toJson(eventsHistory, listOfEvents);
            preferences.edit().putString("EVENTS_HISTORY", strEvents).apply();

    }


    @TargetApi(Build.VERSION_CODES.M)
    private boolean hasPermissions() {
        try {
            if (mActivity != null) {
                for (String permission : mPermissions) {
                    if (mActivity.checkSelfPermission(permission) != PackageManager.PERMISSION_GRANTED
                            || mActivity.checkSelfPermission(permission) == PackageManager.PERMISSION_DENIED
                            && mActivity.shouldShowRequestPermissionRationale(permission)) {
                        return false;
                    }
                }
            }
            return true;
        } catch (Exception ex) {
            return false;
        }

    }


    String getSDKVersion(){
        return MomentsClient.getSDKVersion();
    }

    protected void trackPayloads() {
        payloadReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                count++;
                updatePayloadSentCount();
                mMethodChannel.invokeMethod("onPayloadSent", count);
            }
        };

        LocalBroadcastManager.getInstance(mActivity).registerReceiver((payloadReceiver),
                new IntentFilter(Moments.PAYLOAD_SENT_INTENT)
        );
    }


    protected void trackEngagements() {
        //Receives that handles notification
        NotificationReceiver notificationReceiver = new NotificationReceiver(
                101, R.drawable.lotadata_notification_icon, RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION)) {
            @Override
            public void notificationClicked(Context context, String actionText, String actionLink) {

//                //Compose intent to start activity that contains webview
//                Intent myIntent = new Intent(context, EngagementActivity.class);
//                myIntent.putExtra(Moments.LD_NOTIFICATION_ACTION_TEXT, actionText);
//                myIntent.putExtra(Moments.LD_NOTIFICATION_ACTION_LINK, actionLink);
//                myIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
//
//                context.startActivity(myIntent);
            }
        };

        mMomentsClient.trackEngagements(notificationReceiver);
    }


    /**
     * Get past payload sent count (from  previous launch)
     */
    private int getPastCount() {

        SharedPreferences sharedpreferences = mActivity.getSharedPreferences(PAYLOAD_COUNT_PREFS, Context.MODE_PRIVATE);
        String value = sharedpreferences.getString(PAYLOAD_COUNT_KEY, "0");
        return Integer.valueOf(value);
    }


    /**
     * Update payload send count
     */
    private void updatePayloadSentCount() {
        SharedPreferences sharedpreferences = mActivity.getSharedPreferences(PAYLOAD_COUNT_PREFS, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedpreferences.edit();
        editor.putString(PAYLOAD_COUNT_KEY, String.valueOf(count));
        editor.commit();
    }



    protected void untrackEvents() {
        if(mMomentsClient != null) mMomentsClient.untrackEvents(mEventListener);
    }

    protected void untrackEngagements() {
        if(mMomentsClient != null) mMomentsClient.untrackEngagements();
    }


    public void disposeSdk() {

        LocalBroadcastManager.getInstance(mActivity).unregisterReceiver(payloadReceiver);

        if (mMomentsClient != null) {
            untrackEvents();
            untrackEngagements();
            mMomentsClient.recordEvent("Stop SDK");
            mMomentsClient.disconnect();
        }

        updatePayloadSentCount();

    }
}
